@@ -1,11 +1,20 @@
+from builtins import str
 from binascii import hexlify
 from hashlib import sha512
 import base64
 import json
 import logging
 import re
-import urllib
-import urllib2
+try:
+    # python2
+    from urllib import urlencode
+    from urllib2 import Request, urlopen
+    from urllib2 import HTTPError
+except Exception:
+    # python3
+    from urllib.parse import urlencode
+    from urllib.request import Request, urlopen
+    from urllib.error import HTTPError
 
 from sqlalchemy.orm import joinedload
 from sqlalchemy.orm.exc import (
@@ -43,9 +52,9 @@ def add_role(session,
              is_admin=False):
     try:
         role = Roles(
-            role_name=unicode(role_name),
-            role_password=unicode(role_password),
-            role_email=unicode(role_email),
+            role_name=str(role_name),
+            role_password=str(role_password),
+            role_email=str(role_email),
             is_active=is_active,
             is_admin=is_admin)
         session.add(role)
@@ -74,16 +83,16 @@ def update_role(session,
                 role_phone=None):
     try:
         role = session.query(Roles) \
-            .filter_by(role_name=unicode(role_name)) \
+            .filter_by(role_name=str(role_name)) \
             .first()
         if new_role_name is not None:
-            role.role_name = unicode(new_role_name)
+            role.role_name = str(new_role_name)
         if role_password is not None:
-            role.role_password = unicode(role_password)
+            role.role_password = str(role_password)
         if role_email is not None:
-            role.role_email = unicode(role_email)
+            role.role_email = str(role_email)
         if role_phone is not None:
-            role.role_phone = unicode(role_phone) if role_phone else None
+            role.role_phone = str(role_phone) if role_phone else None
         if is_active is not None:
             role.is_active = is_active
         if is_admin is not None:
@@ -100,7 +109,7 @@ def update_role(session,
                                   (new_role_name))
         else:
             raise TemboardUIError(e.message)
-    except AttributeError as e:
+    except AttributeError:
         raise TemboardUIError(400, "Role '%s' not found." % (role_name))
     except Exception as e:
         raise TemboardUIError(500, e.message)
@@ -110,8 +119,8 @@ def get_role(session, role_name):
     try:
         return session.query(Roles).options(
             joinedload(Roles.groups)).filter_by(
-                role_name=unicode(role_name)).first()
-    except AttributeError as e:
+                role_name=str(role_name)).first()
+    except AttributeError:
         raise TemboardUIError(400, "Role '%s' not found." % (role_name))
     except Exception as e:
         raise TemboardUIError(e.message)
@@ -120,9 +129,9 @@ def get_role(session, role_name):
 def delete_role(session, role_name):
     try:
         role = session.query(Roles).filter(
-            Roles.role_name == unicode(role_name)).one()
+            Roles.role_name == str(role_name)).one()
         session.delete(role)
-    except NoResultFound as e:
+    except NoResultFound:
         raise TemboardUIError(400, "Role '%s' not found." % (role_name))
     except Exception as e:
         raise TemboardUIError(e.message)
@@ -142,9 +151,9 @@ Groups
 def add_group(session, group_name, group_description, group_kind):
     try:
         group = Groups(
-            group_name=unicode(group_name),
-            group_description=unicode(group_description),
-            group_kind=unicode(group_kind))
+            group_name=str(group_name),
+            group_description=str(group_description),
+            group_kind=str(group_kind))
         session.add(group)
         session.flush()
         return group
@@ -165,14 +174,14 @@ def get_group(session, group_name, group_kind):
     try:
         if group_kind == 'role':
             return session.query(Groups).filter_by(
-                group_name=unicode(group_name),
-                group_kind=unicode(group_kind)).one()
+                group_name=str(group_name),
+                group_kind=str(group_kind)).one()
         else:
             return session.query(Groups).options(
                 joinedload(Groups.ari)).filter(
-                    Groups.group_name == unicode(group_name),
-                    Groups.group_kind == unicode(group_kind)).one()
-    except AttributeError as e:
+                    Groups.group_name == str(group_name),
+                    Groups.group_kind == str(group_kind)).one()
+    except AttributeError:
         raise TemboardUIError(400, "Group '%s' (%s) not found." % (group_name,
                                                                    group_kind))
     except Exception as e:
@@ -187,13 +196,13 @@ def update_group(session,
     try:
         group = session.query(Groups) \
             .filter_by(
-                group_name=unicode(group_name),
-                group_kind=unicode(group_kind)) \
+                group_name=str(group_name),
+                group_kind=str(group_kind)) \
             .first()
         if new_group_name is not None:
-            group.group_name = unicode(new_group_name)
+            group.group_name = str(new_group_name)
         if group_description is not None:
-            group.group_description = unicode(group_description)
+            group.group_description = str(group_description)
         session.merge(group)
         session.flush()
         return group
@@ -203,7 +212,7 @@ def update_group(session,
                                   (new_group_name))
         else:
             raise TemboardUIError(400, e.message)
-    except AttributeError as e:
+    except AttributeError:
         raise TemboardUIError(400, "Group '%s' ('%s') not found." %
                               (group_name, group_kind))
     except Exception as e:
@@ -213,10 +222,10 @@ def update_group(session,
 def delete_group(session, group_name, group_kind):
     try:
         group = session.query(Groups).filter(
-            Groups.group_name == unicode(group_name),
-            Groups.group_kind == unicode(group_kind)).one()
+            Groups.group_name == str(group_name),
+            Groups.group_kind == str(group_kind)).one()
         session.delete(group)
-    except NoResultFound as e:
+    except NoResultFound:
         raise TemboardUIError(400, "Group '%s' not found." % (group_name))
     except Exception as e:
         raise TemboardUIError(400, e.message)
@@ -225,18 +234,18 @@ def delete_group(session, group_name, group_kind):
 def get_group_list(session, group_kind='role'):
     if group_kind == 'role':
         return session.query(Groups).filter(
-            Groups.group_kind == unicode(group_kind)).order_by(
+            Groups.group_kind == str(group_kind)).order_by(
                 Groups.group_name).all()
     else:
         return session.query(Groups).options(joinedload(Groups.ari)).filter(
-            Groups.group_kind == unicode(group_kind)).order_by(
+            Groups.group_kind == str(group_kind)).order_by(
                 Groups.group_name).all()
 
 
 def add_role_in_group(session, role_name, group_name):
     try:
         role_group = RoleGroups(
-            role_name=unicode(role_name), group_name=unicode(group_name))
+            role_name=str(role_name), group_name=str(group_name))
         session.add(role_group)
         session.flush()
     except IntegrityError as e:
@@ -258,10 +267,10 @@ def add_role_in_group(session, role_name, group_name):
 def delete_role_from_group(session, role_name, group_name):
     try:
         role_group = session.query(RoleGroups).filter(
-            RoleGroups.group_name == unicode(group_name),
-            RoleGroups.role_name == unicode(role_name)).one()
+            RoleGroups.group_name == str(group_name),
+            RoleGroups.role_name == str(role_name)).one()
         session.delete(role_group)
-    except NoResultFound as e:
+    except NoResultFound:
         raise TemboardUIError(400, "Role '%s' not found in group '%s'." %
                               (role_name, group_name))
     except Exception as e:
@@ -270,7 +279,7 @@ def delete_role_from_group(session, role_name, group_name):
 
 def get_groups_by_role(session, role_name):
     return session.query(RoleGroups).filter(
-        RoleGroups.role_name == unicode(role_name)).order_by(
+        RoleGroups.role_name == str(role_name)).order_by(
             RoleGroups.group_name).all()
 
 
@@ -278,14 +287,14 @@ def get_instance_groups_by_role(session, role_name):
     return session.query(InstanceGroups.group_name).filter(
         InstanceGroups.group_name == AccessRoleInstance.instance_group_name,
         AccessRoleInstance.role_group_name == RoleGroups.group_name,
-        RoleGroups.role_name == unicode(role_name)).group_by(
+        RoleGroups.role_name == str(role_name)).group_by(
             InstanceGroups.group_name).order_by(
                 InstanceGroups.group_name).all()
 
 
 def get_roles_by_group(session, group_name):
     return session.query(Roles).filter(
-        RoleGroups.group_name == unicode(group_name),
+        RoleGroups.group_name == str(group_name),
         Roles.role_name == RoleGroups.role_name).order_by(
             Roles.role_name).all()
 
@@ -310,11 +319,11 @@ def add_instance(session,
                  comment=None):
     try:
         instance = Instances(
-            agent_address=unicode(new_agent_address),
+            agent_address=str(new_agent_address),
             agent_port=int(new_agent_port),
-            hostname=unicode(hostname))
+            hostname=str(hostname))
         if agent_key is not None:
-            instance.agent_key = unicode(agent_key)
+            instance.agent_key = str(agent_key)
         if cpu is not None and cpu != u'':
             instance.cpu = int(cpu)
         if memory_size is not None and memory_size != u'':
@@ -322,11 +331,11 @@ def add_instance(session,
         if pg_port is not None and pg_port != u'':
             instance.pg_port = int(pg_port)
         if pg_version is not None:
-            instance.pg_version = unicode(pg_version)
+            instance.pg_version = str(pg_version)
         if pg_version_summary is not None:
-            instance.pg_version_summary = unicode(pg_version_summary)
+            instance.pg_version_summary = str(pg_version_summary)
         if pg_data is not None:
-            instance.pg_data = unicode(pg_data)
+            instance.pg_data = str(pg_data)
         instance.notify = bool(notify)
         instance.comment = comment
         session.add(instance)
@@ -348,9 +357,9 @@ def get_instance(session, agent_address, agent_port):
         return session.query(Instances).options(
             joinedload(Instances.groups),
             joinedload(Instances.plugins)).filter_by(
-                agent_address=unicode(agent_address),
+                agent_address=str(agent_address),
                 agent_port=agent_port).first()
-    except AttributeError as e:
+    except AttributeError:
         raise TemboardUIError(400, "Instance entry '%s:%s' not found." %
                               (agent_address, agent_port))
     except Exception as e:
@@ -375,11 +384,11 @@ def update_instance(session,
     try:
         instance = session.query(Instances) \
             .filter_by(
-                agent_address=unicode(agent_address),
+                agent_address=str(agent_address),
                 agent_port=agent_port) \
             .first()
         if new_agent_address is not None:
-            instance.agent_address = unicode(new_agent_address)
+            instance.agent_address = str(new_agent_address)
         if new_agent_port is not None:
             instance.agent_port = int(new_agent_port)
         if cpu is not None and cpu != u'':
@@ -400,7 +409,7 @@ def update_instance(session,
         for prop in ['agent_key', 'hostname', 'pg_version',
                      'pg_version_summary', 'pg_data']:
             if locals().get(prop) is not None:
-                setattr(instance, prop, unicode(locals().get(prop)))
+                setattr(instance, prop, str(locals().get(prop)))
         session.merge(instance)
         session.flush()
         return instance
@@ -411,7 +420,7 @@ def update_instance(session,
                                   (agent_address, agent_port))
         else:
             raise TemboardUIError(400, e.message)
-    except AttributeError as e:
+    except AttributeError:
         raise TemboardUIError(400, "Instance entry ('%s:%s') not found." %
                               (agent_address, agent_port))
     except Exception as e:
@@ -426,10 +435,10 @@ def delete_instance(session, agent_address, agent_port):
 
     try:
         instance = session.query(Instances).filter(
-            Instances.agent_address == unicode(agent_address),
+            Instances.agent_address == str(agent_address),
             Instances.agent_port == agent_port).one()
         session.delete(instance)
-    except NoResultFound as e:
+    except NoResultFound:
         raise TemboardUIError(400, "Instance entry ('%s:%s') not found." %
                               (agent_address, agent_port))
     except Exception as e:
@@ -444,7 +453,7 @@ def delete_instance(session, agent_address, agent_port):
                 MonitoringHost.hostname == instance.hostname,
                 MonitoringInstance.port == instance.pg_port).one()
         session.delete(monitoring_instance)
-    except NoResultFound as e:
+    except NoResultFound:
         pass
     except Exception as e:
         raise TemboardUIError(400, e.message)
@@ -463,7 +472,7 @@ def delete_instance(session, agent_address, agent_port):
             session.query(MonitoringHost) \
                 .filter(MonitoringHost.hostname == instance.hostname) \
                 .delete()
-        except NoResultFound as e:
+        except NoResultFound:
             pass
         except Exception as e:
             raise TemboardUIError(400, e.message)
@@ -472,14 +481,14 @@ def delete_instance(session, agent_address, agent_port):
 def add_instance_in_group(session, agent_address, agent_port, group_name):
     try:
         # Create instance group if not exists
-        group = Groups(group_name=unicode(group_name), group_kind=u'instance')
+        group = Groups(group_name=str(group_name), group_kind=u'instance')
         session.merge(group)
         session.flush()
 
         instance_group = InstanceGroups(
-            agent_address=unicode(agent_address),
+            agent_address=str(agent_address),
             agent_port=agent_port,
-            group_name=unicode(group_name))
+            group_name=str(group_name))
         session.add(instance_group)
         session.flush()
     except IntegrityError as e:
@@ -503,7 +512,7 @@ def add_instance_in_group(session, agent_address, agent_port, group_name):
 def purge_instance_plugins(session, agent_address, agent_port):
     try:
         plugins = session.query(Plugins).filter(
-            Plugins.agent_address == unicode(agent_address),
+            Plugins.agent_address == str(agent_address),
             Plugins.agent_port == agent_port).all()
         for plugin in plugins:
             session.delete(plugin)
@@ -514,9 +523,9 @@ def purge_instance_plugins(session, agent_address, agent_port):
 def add_instance_plugin(session, agent_address, agent_port, plugin_name):
     try:
         plugin = Plugins(
-            agent_address=unicode(agent_address),
+            agent_address=str(agent_address),
             agent_port=agent_port,
-            plugin_name=unicode(plugin_name))
+            plugin_name=str(plugin_name))
         session.add(plugin)
         session.flush()
     except IntegrityError as e:
@@ -543,11 +552,11 @@ def get_instance_list(
 def delete_instance_from_group(session, agent_address, agent_port, group_name):
     try:
         instance_group = session.query(InstanceGroups).filter(
-            InstanceGroups.agent_address == unicode(agent_address),
+            InstanceGroups.agent_address == str(agent_address),
             InstanceGroups.agent_port == agent_port,
-            InstanceGroups.group_name == unicode(group_name)).one()
+            InstanceGroups.group_name == str(group_name)).one()
         session.delete(instance_group)
-    except NoResultFound as e:
+    except NoResultFound:
         raise TemboardUIError(
             400, "Instance entry ('%s:%s)' not found in group '%s'." %
             (agent_address, agent_port, group_name))
@@ -558,7 +567,7 @@ def delete_instance_from_group(session, agent_address, agent_port, group_name):
 def get_instances_by_group(session, group_name):
     return session.query(Instances).options(
         joinedload(Instances.groups), joinedload(Instances.plugins)).filter(
-            InstanceGroups.group_name == unicode(group_name),
+            InstanceGroups.group_name == str(group_name),
             Instances.agent_address == InstanceGroups.agent_address,
             Instances.agent_port == InstanceGroups.agent_port).order_by(
                 Instances.agent_address).all()
@@ -566,7 +575,7 @@ def get_instances_by_group(session, group_name):
 
 def get_groups_by_instance(session, agent_address, agent_port):
     return session.query(InstanceGroups).filter(
-        InstanceGroups.agent_address == unicode(agent_address),
+        InstanceGroups.agent_address == str(agent_address),
         InstanceGroups.agent_port == agent_port).order_by(
             InstanceGroups.group_name).all()
 
@@ -589,8 +598,8 @@ def add_role_group_in_instance_group(session, role_group_name,
                                      instance_group_name):
     try:
         ari = AccessRoleInstance(
-            role_group_name=unicode(role_group_name),
-            instance_group_name=unicode(instance_group_name))
+            role_group_name=str(role_group_name),
+            instance_group_name=str(instance_group_name))
         session.add(ari)
         session.flush()
     except IntegrityError as e:
@@ -615,11 +624,11 @@ def delete_role_group_from_instance_group(session, role_group_name,
                                           instance_group_name):
     try:
         ari = session.query(AccessRoleInstance).filter(
-            AccessRoleInstance.role_group_name == unicode(role_group_name),
-            AccessRoleInstance.instance_group_name == unicode(
+            AccessRoleInstance.role_group_name == str(role_group_name),
+            AccessRoleInstance.instance_group_name == str(
                 instance_group_name)).one()
         session.delete(ari)
-    except NoResultFound as e:
+    except NoResultFound:
         raise TemboardUIError(
             400, "Role group '%s' not found in instance group '%s'." %
             (role_group_name, instance_group_name))
@@ -634,7 +643,7 @@ def get_instances_by_role_name(session, role_name):
             InstanceGroups.group_name ==
             AccessRoleInstance.instance_group_name,
             AccessRoleInstance.role_group_name == RoleGroups.group_name,
-            RoleGroups.role_name == unicode(role_name)).order_by(
+            RoleGroups.role_name == str(role_name)).order_by(
                 InstanceGroups.group_name, Instances.agent_address)
 
 
@@ -645,8 +654,8 @@ def role_name_can_access_instance(session, role_name, agent_address,
             AccessRoleInstance.instance_group_name ==
             InstanceGroups.group_name,
             AccessRoleInstance.role_group_name == RoleGroups.group_name,
-            RoleGroups.role_name == unicode(role_name),
-            InstanceGroups.agent_address == unicode(agent_address),
+            RoleGroups.role_name == str(role_name),
+            InstanceGroups.agent_address == str(agent_address),
             InstanceGroups.agent_port == agent_port).one()
     except (NoResultFound, Exception):
         raise TemboardUIError(400, "You don't have access to this instance.")
@@ -655,13 +664,13 @@ def role_name_can_access_instance(session, role_name, agent_address,
 def get_role_by_auth(session, role_name, role_password):
     try:
         role = session.query(Roles).filter(
-            Roles.role_name == unicode(role_name),
+            Roles.role_name == str(role_name),
             Roles.is_active.is_(True)).one()
-        if role.role_password != unicode(role_password):
+        if role.role_password != str(role_password):
             raise TemboardUIError(400, "Wrong user/password: %s/%s" %
                                   (role_name, role_password))
         return role
-    except NoResultFound as e:
+    except NoResultFound:
         raise TemboardUIError(400, "Wrong user/password: %s/%s" %
                               (role_name, role_password))
     except Exception as e:
@@ -712,12 +721,12 @@ def get_role_by_cookie(session, content):
             raise Exception("Cookie's content is corrupted.")
         try:
             role = session.query(Roles).filter(
-                Roles.role_name == unicode(c_role_name),
+                Roles.role_name == str(c_role_name),
                 Roles.is_active.is_(True)).one()
         except (NoResultFound, Exception):
             raise Exception("Role '%s' not found or not active." %
                             (c_role_name))
-        if role.role_password[:15] != unicode(c_fp_hash_password):
+        if role.role_password[:15] != str(c_fp_hash_password):
             raise Exception("Password sign not correct.")
         return role
     else:
@@ -821,13 +830,13 @@ def send_sms(config, content, phones):
 
     errors = []
     for recipient in phones:
-        req = urllib2.Request(url=uri)
+        req = Request(url=uri)
         req.add_header('Authorization', 'Basic %s' % s)
         data = {'From': from_, 'Body': content, 'To': recipient}
-        req.add_data(urllib.urlencode(data))
+        req.add_data(urlencode(data))
         try:
-            urllib2.urlopen(req)
-        except urllib2.HTTPError as e:
+            urlopen(req)
+        except HTTPError as e:
             response = json.loads(e.read())
             logger.error("Could not send SMS; %s" % response.get('message'))
             errors.append(recipient)
